#!/usr/bin/env python
# -*- coding: utf-8 -*-
#       audio.py
#
#       Copyright 2008 Gordon Allott <gordallott@gmail.com>
#
#       This program is free software; you can redistribute it and/or modify
#       it under the terms of the GNU General Public License as published by
#       the Free Software Foundation; either version 3 of the License, or
#       (at your option) any later version.
#
#       This program is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#       GNU General Public License for more details.
#
#       You should have received a copy of the GNU General Public License
#       along with this program; if not, write to the Free Software
#       Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#       MA 02110-1301, USA.
#
#       #
import os, sys, math, random
from numpy import *
import pygame
from pygame.locals import *
import random
from math import *
from gamelib.util import *

import data as Data

class AudioSystem:

    sounds = {}
    num_channels = 32
    high = 0
    immediate = 0

    channels = []

    clock = None
    ctime = 0

    def __init__(self, num_channels=32, high_reserved=5, immediate_reserved=3):
        pygame.mixer.init(44100, -16, 2, 1024)

        self.num_channels = 32
        pygame.mixer.set_num_channels(self.num_channels)
        self.clock = pygame.time.Clock()
        self.clock.tick()

        self.channel_samples = []
        self.high = high_reserved
        self.immediate = immediate_reserved

        for index in range(self.num_channels):
            nchannel = Channel(index)
            self.channels.append(nchannel)


    def step(self):
        self.cleanup()

    def stop_all(self):

        for channel in self.channels:
            try:
                channel.stop()
            except:
                pass

    def create_sound(self, filename):
        if self.sounds.has_key(filename):
            return self.sounds[filename]

        sound = Sample(filename)
        self.sounds[filename] = sound

        sound.sound.set_volume(0.5)
        return sound


    def debug_channels(self):
        free_channels = 0
        for channel in self.channels:
            #print 'id', channel.id, 'is free?', channel.is_free, 'when free?', channel.when_free
            if channel.is_free:
                free_channels += 1

        print 'num_free channels:', free_channels


    def cleanup(self):
        self.clock.tick()
        self.ctime += self.clock.get_rawtime() / 1000.0
        ctime = self.ctime

        for channel in self.channels:
            if not channel.is_free:
                if ctime > channel.when_free:
                    channel.stop()

    def get_channel(self, priority='normal'):
        normal_channels = self.num_channels - self.high - self.immediate
        self.clock.tick()
        ctime = self.clock.get_rawtime() / 1000.0


        if priority == 'normal':
            for index in range(0, normal_channels):
                channel = self.channels[index]

                if channel.is_free:
                    return channel

        if priority == 'high':
            for index in range(normal_channels, self.high):
                channel = self.channels[index]

                if channel.is_free:
                    return channel

            # no high priority channels, try normal ones
            return self.get_channel(priority='normal')

        if priority == 'immediate':
            for index in range(normal_channels + self.high, self.immediate):
                channel = self.channels[index]

                if channel.is_free:
                    return channel

            # no immediate channels, try normal ones
            channel = self.get_channel(priority='normal')

            if channel:
                return channel

            # no normal ones, try high priority
            channel = self.get_channel(priority='high')

            if channel:
                return channel

        return None

    def play(self, sound, priority='normal'):

        #self.clock.tick()

        channel = self.get_channel(priority=priority)
        if not channel:
            raise Exception, 'no channels left :('

        # todo - immediate priority needs to play immediatly on any channel
        # if one can't be found - guarenteed even if it meens overwriting an
        # immediate channel

        # high priority needs to overwrite a normal channel if it can't find
        # a high priority one
        playnow = False
        channel.play(sound.get_raw(), ctime=self.ctime, now=playnow)
        return channel

    def play_music(self, filename):
        filename = Data.filepath(os.path.join('audio/', filename))
        pygame.mixer.music.load(filename)
        pygame.mixer.music.play(-1)

    def stop_music(self, fadeout=None):
        if fadeout:
            pygame.mixer.music.fadeout(fadeout)
        else:
            pygame.mixer.music.stop()


class Channel:

    sample = None

    is_free = True          # if the channel is free for a new sound
    when_free = None        # when the sound finishes out or None if its already free

    id = 0
    _channel = None

    clock = None

    def __init__(self, channel_id):
        self.sample = None
        self.is_free = True
        self.when_free = None
        self.id = channel_id

        self._channel = pygame.mixer.Channel(channel_id)



    def play(self, sound, ctime=0, now=False):
        if now and not self.is_free:
            self.stop()

        if not self.is_free:
            raise Exception, 'channel is not free'

        sound.set_volume(1.0)
        self._channel.set_volume(0.5)

        slength = sound.get_length()
        self.when_free = ctime + slength
        self.sample = sound
        self.is_free = False

        self._channel.play(self.sample)


    def stop(self, fadeout=False):
        try:
            if fadeout:
                self._channel.fadeout(fadeout)
            else:
                self._channel.stop()
        except:
            pass

        self.sample = None
        self.is_free = True
        self.when_free = None



class Sample:

    sound = None

    def __init__(self, filename):

        filepath = Data.filepath(os.path.join('audio/', filename))
        if not os.path.exists(filepath):
            raise Exception, 'path does not exist'

        self.sound = pygame.mixer.Sound(filepath)

    def play(self):
        self.sound.play()

    def get_raw(self):
        return self.sound
