#!/usr/bin/env python
# -*- coding: utf-8 -*-
#       intro.py
#
#       Copyright 2008 Gordon Allott <gordallott@gmail.com>
#
#       This program is free software; you can redistribute it and/or modify
#       it under the terms of the GNU General Public License as published by
#       the Free Software Foundation; either version 3 of the License, or
#       (at your option) any later version.
#
#       This program is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#       GNU General Public License for more details.
#
#       You should have received a copy of the GNU General Public License
#       along with this program; if not, write to the Free Software
#       Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#       MA 02110-1301, USA.
#
#
import pygame
from pygame.locals import *

import gamelib.data
import gamelib.render
import gamelib.world.gameworld
import gamelib.timeline
from gamelib.util import *
from gamelib.timeline import time_now, time_start
from gamelib.world.props import *

from scene import *


class intro_scene(Scene):

    intro_images = [
        ('intro_logo', 1.0, 1.6, 6.0, 8.0, 0.5),
        ('intro_slide1.png', 0.5, 1.5, 8.0, 10.0, 1.0),
        ('intro_slide2.png', 0.0, 2.0, 8.0, 10.0, 1.0),
    ]

    c_image = 0
    c_sprite = None

    def __init__(self, *args, **kwargs):
        Scene.__init__(self, *args, **kwargs)

        self.timeline = gamelib.timeline.Timeline()
        gamelib.instance.instance.audio.play_music('Somewhere Sunny.ogg')



        #preload our textures
        for intro in self.intro_images:
            texture = self.render.create_texture(intro[0], repeat=False, fps=8)

        texture = self.render.create_texture(self.intro_images[0][0], repeat=False, fps=8)
        sprite_size = texture.size[0] / 800.0
        sprite_size = (sprite_size, sprite_size * texture.aspect)
        self.c_sprite = gamelib.render.Sprite(self.render, texture, size=(sprite_size[0] * 10.0, sprite_size[1] * 10.0))

        #add out timeline changes
        ctime = 0.0
        #for intro in self.intro_images:
            #fn, in_start, in_end, out_start, out_end = intro
            #in_length = in_end - in_start
            #out_length = out_end - out_start
            #in_wait = in_start
            #out_wait = out_start - in_length
            #total = in_wait + out_wait + out_length

            #self.timeline.add_action(time=ctime + in_wait, relative=time_now,
                                     #repeat_callback=True, length=in_length,
                                     #callback=self.intro_images_in)

            #self.timeline.add_action(time=ctime + in_wait + out_wait,
                                     #relative=time_now,
                                     #repeat_callback=True, length=out_length,
                                     #callback=self.intro_images_out)

            #self.timeline.add_action(time=ctime + in_wait + out_wait + out_length,
                                     #relative=time_now,
                                     #repeat_callback=False,
                                     #callback=self.change_image)

            #ctime += total



        self.do_image_change = True


        self.input.add_key(K_RETURN, self.cb_skip, repeating=False)
        self.cimage = 0


    def cb_skip(self, key):
        self.stop_scene = True

    def step(self):
        if self.joystick:
            dojump = False

            buttons = self.joystick.get_numbuttons()
            for index in range(buttons):
                if self.joystick.get_button(index):
                    dojump = True

            if dojump:
                self.cb_skip(K_w)

        if self.do_image_change:
            fn, in_start, in_end, out_start, out_end, size = self.intro_images[self.cimage]
            in_length = in_end - in_start
            out_length = out_end - out_start
            in_wait = in_start
            out_wait = out_start - in_length
            total = in_wait + out_wait + out_length


            self.timeline.add_action(time=in_wait, relative=time_now,
                                     repeat_callback=True, length=in_length,
                                     callback=self.intro_images_in)

            self.timeline.add_action(time=in_wait + out_wait,
                                     relative=time_now,
                                     repeat_callback=True, length=out_length,
                                     callback=self.intro_images_out)

            self.timeline.add_action(time=in_wait + out_wait + out_length,
                                     relative=time_now,
                                     callback=self.change_image)

            texture = self.render.create_texture(self.intro_images[self.c_image][0], repeat=False, fps=12)
            sprite_size = texture.size[0] / 800.0
            sprite_size = (sprite_size, sprite_size * texture.aspect)
            self.c_sprite = gamelib.render.Sprite(self.render, texture, size=(sprite_size[0] * 20.0 * self.intro_images[self.c_image][5], sprite_size[1] * 20.0 * self.intro_images[self.c_image][5]))
            self.c_sprite.color = (1.0, 1.0, 1.0, 0.0)

            self.do_image_change = False

        self.timeline.step()
        self.render.clearscreen()

        self.c_sprite.position = (0.0, 0.0)
        self.c_sprite.draw()

    def intro_images_in(self, progress):
        fade = cosine_interpolate(0.0, 1.0, progress)
        self.c_sprite.color = (1.0, 1.0, 1.0, fade)

    def intro_images_out(self, progress):
        fade = cosine_interpolate(1.0, 0.0, progress)
        self.c_sprite.color = (1.0, 1.0, 1.0, fade)

    def change_image(self, progress):
        self.do_image_change = True
        self.c_image += 1
        if self.c_image >= len(self.intro_images):
            self.stop_scene = True
            return

    def shutdown(self):
        pass


    def cb_jump(self, key):
        self.cb_skip(key)
