#!/usr/bin/env python
# -*- coding: utf-8 -*-
#       level_one.py
#
#       Copyright 2008 Gordon Allott <gordallott@gmail.com>
#
#       This program is free software; you can redistribute it and/or modify
#       it under the terms of the GNU General Public License as published by
#       the Free Software Foundation; either version 3 of the License, or
#       (at your option) any later version.
#
#       This program is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#       GNU General Public License for more details.
#
#       You should have received a copy of the GNU General Public License
#       along with this program; if not, write to the Free Software
#       Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#       MA 02110-1301, USA.
#
#


import pygame
from pygame.locals import *

import gamelib.data
import gamelib.render
import gamelib.world.gameworld
from gamelib.world.props import *
from gamelib.world.actors import *
from gamelib.world.gameworld import hex_to_gl
import gamelib.instance
from gamelib.timeline import Timeline, time_now, time_start
from gamelib.util import cosine_interpolate, linear_interpolate
from scene import *
from OpenGL.GL import *


class level_six(Scene):

    level_geom = 'levels/level_six.svg'

    checkpoints = [(-4.0, 1.27*0.5),(4.0, 7.27*0.5),(20.0, 1.27*0.5)]

    def __init__(self, *args, **kwargs):
        Scene.__init__(self, *args, **kwargs)
        self.timeline = Timeline()
        gamelib.instance.instance.audio.play_music('Interloper.ogg')

        self.start_level()

    def start_level(self,checkpoint=0):
        self.fade_value = 0.0
        self.fade_blend = 'addictive'
        self.do_fade = False

        self.do_reset = False

        self.stop_scene = False
        self.game_world = gamelib.world.gameworld.world(self.render)
        #self.game_world.base_ambiant = (0.2, 0.2, 0.2)
        #self.game_world.bg_ambiant = (0.3, 0.3, 0.3)
        gamelib.instance.instance.audio.stop_all()

        # add an actor
        #self.player = self.game_world.add_actor(actor_object=Player, position=(-4.0, 1.27*0.5))
        #self.player = self.game_world.add_actor(actor_object=Player, position=(4.0, 7.27*0.5))
        #self.player = self.game_world.add_actor(actor_object=Player, position=(20.0, 1.27*0.5))
        #self.player = self.game_world.add_actor(actor_object=Player, position=(6.0, 26.27*0.5))
        self.player = self.game_world.add_actor(actor_object=Player, position=(-5, 5))

        self.set_keys()

        self.build_geoms(self.game_world, 'levels/level_six.svg')

        layer = self.game_world.create_layer(pos=-5, relative_movement=(0.25, 0.8))
        layer.add_graphic('bg2.png', tex_size=(20.0, 10.0), height=20.0, y_position=1.0)


        self.fade_value = 1.0
        self.fade_blend = 'addictive'
        self.do_fade = True

        self.reset_scene = False

        self.game_world.optimise_geoms()
        self.game_world.optimise_back_geoms()
        self.game_world.bg_gradient = (hex_to_gl('#fed853'), hex_to_gl('#f1382c'))
        self.timeline.add_action(time=0.0, length=2.0, relative=time_now,
                                     repeat_callback=True,
                                     callback=self.timeline_cb_fadein_white)



