#!/usr/bin/env python
# -*- coding: utf-8 -*-
#       intro.py
#
#       Copyright 2008 Gordon Allott <gordallott@gmail.com>
#
#       This program is free software; you can redistribute it and/or modify
#       it under the terms of the GNU General Public License as published by
#       the Free Software Foundation; either version 3 of the License, or
#       (at your option) any later version.
#
#       This program is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#       GNU General Public License for more details.
#
#       You should have received a copy of the GNU General Public License
#       along with this program; if not, write to the Free Software
#       Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#       MA 02110-1301, USA.
#
#
import pygame
from pygame.locals import *

import gamelib.data
import gamelib.render
import gamelib.world.gameworld
import gamelib.timeline
from gamelib.timeline import time_now, time_start
from gamelib.world.props import *

from scene import *


class menu_scene(Scene):

    start_img = None

    def __init__(self, *args, **kwargs):
        Scene.__init__(self, *args, **kwargs)

        self.timeline = gamelib.timeline.Timeline()

        tex = self.render.create_texture('sprites/menu_start.png', repeats=('clamp', 'clamp'))
        self.start_img = gamelib.render.Sprite(self.render, tex, size=(4.0, 1.0),
                                               blend='modulate')

        tex = self.render.create_texture('layered/gradient_colours/bg2.png', repeats=('clamp', 'clamp'))
        self.img1 = gamelib.render.Sprite(self.render, tex, size=(20*self.render.aspect, 6.0), tile_repeats=(1.0, 1.0),
                                               blend='modulate')


        tex = self.render.create_texture('sprites/Title.png', repeats=('clamp', 'clamp'))
        self.title = gamelib.render.Sprite(self.render, tex, size=(4.0, 0.5), tile_repeats=(1.0, 1.0),
                                               blend='modulate')

        tex = self.render.create_texture('info1.png', repeats=('clamp', 'clamp'))
        self.info = gamelib.render.Sprite(self.render, tex, size=(8.0, 2.0), tile_repeats=(1.0, 1.0),
                                               blend='subtractive')
        self.info.color = (0.0, 0.0, 0.0, 0.0)


        ##add out timeline changes
        #ctime = 0.0
        #for intro in self.intro_images:
            #fn, in_start, in_end, out_start, out_end = intro
            #in_length = in_end - in_start
            #out_length = out_end - out_start
            #in_wait = in_start
            #out_wait = out_start - in_length
            #total = in_wait + out_wait + out_length

            #self.timeline.add_action(time=ctime + in_wait, relative=time_now,
                                     #repeat_callback=True, length=in_length,
                                     #callback=self.intro_images_in)

            #self.timeline.add_action(time=ctime + in_wait + out_wait,
                                     #relative=time_now,
                                     #repeat_callback=True, length=out_length,
                                     #callback=self.intro_images_out)

            #self.timeline.add_action(time=ctime + in_wait + out_wait + out_length,
                                     #relative=time_now,
                                     #repeat_callback=False,
                                     #callback=self.change_image)

            #ctime += total


        self.fade_value = 1.0
        self.fade_blend = 'addictive'
        self.do_fade = True

        self.timeline.add_action(time=0.0, length=2.0, relative=time_now,
                                 repeat_callback=True,
                                 callback=self.timeline_cb_fadein_black)

        self.timeline.add_action(time=4.0, length=4.0, relative=time_now, repeat_action=True,
                                 repeat_callback=True, callback=self.pulse_start)

        self.timeline.add_action(time=2.0, length=2.0, relative=time_now,
                                 repeat_callback=True, callback=self.start_fade)

        self.input.add_key(K_RETURN, self.cb_skip, repeating=False)

        self.start_img.color = (1.0, 1.0, 1.0, 0.0)



    def draw_bars(self):
        scale = self.render.base_scalar
        height = 5.0

        self.render.reset()
        self.render.disable(GL_TEXTURE_2D)
        self.render.disable(GL_BLEND)

        glBegin(GL_QUADS)

        glColor4fv((0.2, 0.2, 0.2, 1.0))

        glVertex3f(-30.0, -height, -1.0)
        glVertex3f( 30.0, -height, -1.0)
        glVertex3f( 30.0, -height + 3.0, -1.0)
        glVertex3f(-30.0, -height + 3.0, -1.0)

        glVertex3f(-100.0, height, -1.0)
        glVertex3f( 100.0, height, -1.0)
        glVertex3f( 100.0, height - 1.0, -1.0)
        glVertex3f(-100.0, height - 1.0, -1.0)

        glColor4fv((1.0, 0.6, 0.2, 1.0))
        glVertex3f(-100.0, height - 1.0, -1.0)
        glVertex3f( 100.0, height - 1.0, -1.0)
        glVertex3f( 100.0, -height + 3.0, -1.0)
        glVertex3f(-100.0, -height + 3.0, -1.0)

        glEnd()

    def cb_skip(self, key):

        self.timeline.add_action(time=0.0, length=2.0, relative=time_now,
                                 repeat_callback=True,
                                 callback=self.timeline_cb_fadeout_white)

        self.timeline.add_action(time=3.0, length=2.0, relative=time_now,
                                 repeat_callback=True,
                                 callback = self.info_fadein)

        self.timeline.add_action(time=8.0, length=2.0, relative=time_now,
                                 repeat_callback=True,
                                 callback = self.info_fadeout)

        self.timeline.add_action(time=10.0, relative=time_now,
                                 callback=self.cb_stop)

        gamelib.instance.instance.audio.stop_music(fadeout=2000)

    def cb_stop(self, progress):
        self.stop_scene = True

    def step(self):
        if self.joystick:
            dojump = False

            buttons = self.joystick.get_numbuttons()
            for index in range(buttons):
                if self.joystick.get_button(index):
                    dojump = True

            if dojump:
                self.cb_skip(K_w)

        self.timeline.step()
        #self.draw_fs(fade=0.6, blend='replace')

        self.draw_bars()

        self.start_img.position = (0.0, -3.5)
        self.start_img.draw()
        self.img1.position = (0.0, 1.0)
        self.img1.draw()

        self.title.position = (0.0, -2.5)
        self.title.draw()

        if self.do_fade:
            self.draw_fs(fade=self.fade_value, blend=self.fade_blend)

        self.info.position = (0.0, 0.0)
        self.info.draw()

    def timeline_cb_fadein_black(self, position):
        self.fade_value = cosine_interpolate(1.0, 0.0, position)
        self.fade_blend = 'subtractive'
        self.do_fade = True


    def timeline_cb_fadeout_white(self, position):
        self.fade_value = cosine_interpolate(0.0, 1.0, position)
        self.fade_blend = 'addictive'
        self.do_fade = True

    def info_fadein(self, position):
        fade = cosine_interpolate(0.0, 1.0, position)
        self.info.color = (fade, fade, fade, 1.0)

    def info_fadeout(self, position):
        fade = cosine_interpolate(1.0, 0.0, position)
        self.info.color = (fade, fade, fade, 1.0)


    def pulse_start(self, progress):

        fade = abs(-1.0 + (2.0 * progress))
        fade = cosine_interpolate(0.5, 1.0, fade)
        self.start_img.color = (1.0, 1.0, 1.0, fade)

    def start_fade(self, progress):
        fade = cosine_interpolate(0.0, 1.0, progress)
        self.start_img.color = (1.0, 1.0, 1.0, fade)

    def shutdown(self):
        pass

    def cb_jump(self, key):
        self.cb_skip(key)
