#!/usr/bin/env python
# -*- coding: utf-8 -*-
#       scene.py
#
#       Copyright 2008 Gordon Allott <gordallott@gmail.com>
#
#       This program is free software; you can redistribute it and/or modify
#       it under the terms of the GNU General Public License as published by
#       the Free Software Foundation; either version 3 of the License, or
#       (at your option) any later version.
#
#       This program is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#       GNU General Public License for more details.
#
#       You should have received a copy of the GNU General Public License
#       along with this program; if not, write to the Free Software
#       Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#       MA 02110-1301, USA.
#
#       #
import gamelib.data as Data
import pygame
from pygame.locals import *
import random
from math import *
from xml.dom import minidom
import os
from gamelib.world.props import *
from gamelib.world.actors import *
from OpenGL.GL import *
from gamelib.util import *

class Camera:

    def __init__(self, pos=(0.0, 0.0)):
        self.x, self.y = pos
        self.x_vel = 0.0
        self.y_vel = 0.0

    def move_to(self, pos):
        px, py = pos

        dx = px - self.x
        dy = py - self.y

        self.x_vel = dx * 0.3
        self.y_vel = dy * 0.3

        self.x += self.x_vel
        self.y += self.y_vel

        self.x_vel *= 0.9
        self.y_vel *= 0.9

        return (self.x, self.y)

class Scene:

    stop_scene = False

    def _parse_attr(self, attr):
        begin = attr.find('(')
        end = attr.find(')')
        attr_name = attr[0:begin]
        begin += 1
        vals = str(attr[begin:end]).split(',')
        for val in vals:
            val = int(val)
        ret_val = {}
        ret_val[attr_name] = vals
        return ret_val

    def build_geoms(self, gameworld, filename):
        filepath = Data.filepath(filename)
        xml_doc = minidom.parse(filepath)

        root = xml_doc.getElementsByTagName('svg')[0]
        screen_width = float(root.attributes['width'].value)
        screen_height = float(root.attributes['height'].value)

        layers = xml_doc.getElementsByTagName('g')

        # Get level geoms
        for layer in layers:
            if layer.attributes['inkscape:label'].value == "LEVEL_GEOM":
                xml_geoms = layer.getElementsByTagName('image')


        for tag in xml_geoms:
            filename = os.path.split(tag.attributes['xlink:href'].value)[1]


            width = float(tag.attributes['width'].value)
            height = float(tag.attributes['height'].value)
            x = float(tag.attributes['x'].value)
            y = screen_height - (float(tag.attributes['y'].value) + height)
            doflip_h = False
            doflip_v = False
            try:
                moo = tag.attributes['transform'].value.encode('utf-8')
                moo = moo.replace('scale(','').replace(')', '').split(',')
                if float(moo[0]) < 0:
                    doflip_h = True

                if float(moo[1]) < 0:
                    y -= screen_height
                    y = screen_height - y - 1.0
                    doflip_v = True

            except Exception, e:
                pass


            actual_x = x + (width * 0.5)
            actual_y = y + (height * 0.5)

            if doflip_h:
                actual_x = -actual_x

            #if doflip_v:
                #print actual_y
                #actual_y = -actual_y - 1.0
                #print actual_y

            geomobj = gameworld.add_geom(size=(width, height), position=(actual_x, actual_y),
                               graphic_filename=filename,
                               tiles=(width, height))


            geomobj.set_flip(doflip_h, doflip_v)


        xml_geoms = []

        # Get level geoms
        for layer in layers:
            if layer.attributes['inkscape:label'].value == "LEVEL_GEOM_NOCOLLIDE":
                xml_geoms = layer.getElementsByTagName('image')


        for tag in xml_geoms:
            filename = os.path.split(tag.attributes['xlink:href'].value)[1]


            width = float(tag.attributes['width'].value)
            height = float(tag.attributes['height'].value)
            x = float(tag.attributes['x'].value)
            y = screen_height - (float(tag.attributes['y'].value) + height)
            doflip_h = False
            doflip_v = False
            try:
                moo = tag.attributes['transform'].value.encode('utf-8')
                moo = moo.replace('scale(','').replace(')', '').split(',')
                if float(moo[0]) < 0:
                    doflip_h = True

                if float(moo[1]) < 0:
                    y -= screen_height
                    y = screen_height - y - 1.0
                    doflip_v = True

            except Exception, e:
                pass



            actual_x = x + (width * 0.5)
            actual_y = y + (height * 0.5)

            if doflip_h:
                actual_x = -actual_x

            #if doflip_v:
                #actual_y = -actual_y - 1.0

            geomobj = gameworld.add_geom(size=(width, height), position=(actual_x, actual_y),
                               graphic_filename=filename,
                               tiles=(width, height), no_collide=True)


            geomobj.set_flip(doflip_h, doflip_v)

        xml_geoms = []

        # Get level geoms
        for layer in layers:
            if layer.attributes['inkscape:label'].value == "LEVEL_DECORATE":
                xml_geoms = layer.getElementsByTagName('image')


        for tag in xml_geoms:
            filename = os.path.split(tag.attributes['xlink:href'].value)[1]


            width = float(tag.attributes['width'].value)
            height = float(tag.attributes['height'].value)
            x = float(tag.attributes['x'].value)
            y = screen_height - (float(tag.attributes['y'].value) + height)

            doflip_h = False
            doflip_v = False
            try:
                moo = tag.attributes['transform'].value.encode('utf-8')
                moo = moo.replace('scale(','').replace(')', '').split(',')
                if float(moo[0]) < 0:
                    doflip_h = True

                if float(moo[1]) < 0:
                    y -= screen_height
                    y = screen_height - y - 1.0
                    doflip_v = True

            except Exception, e:
                pass



            actual_x = x + (width * 0.5)
            actual_y = y + (height * 0.5)

            if doflip_h:
                actual_x = -actual_x

            #if doflip_v:
                #actual_y = -actual_y - 1.0

            geomobj = gameworld.add_geom(size=(width, height), position=(actual_x, actual_y),
                               graphic_filename=filename,
                               tiles=(width, height),
                               no_collide=True,
                               no_detect=True,
                               draw_infront=False)


            geomobj.set_flip(doflip_h, doflip_v)


        xml_geoms = []

        # Get level geoms
        for layer in layers:
            if layer.attributes['inkscape:label'].value == "LEVEL_DECORATE_ABOVE":
                xml_geoms = layer.getElementsByTagName('image')


        for tag in xml_geoms:
            filename = os.path.split(tag.attributes['xlink:href'].value)[1]

            width = float(tag.attributes['width'].value)
            height = float(tag.attributes['height'].value)
            x = float(tag.attributes['x'].value)
            y = screen_height - (float(tag.attributes['y'].value) + height)

            doflip_h = False
            doflip_v = False
            try:
                moo = tag.attributes['transform'].value.encode('utf-8')
                moo = moo.replace('scale(','').replace(')', '').split(',')
                if float(moo[0]) < 0:
                    doflip_h = True

                if float(moo[1]) < 0:
                    y -= screen_height
                    y = screen_height - y - 1.0
                    doflip_v = True

            except Exception, e:
                pass



            actual_x = x + (width * 0.5)
            actual_y = y + (height * 0.5)

            if doflip_h:
                actual_x = -actual_x

            #if doflip_v:
                #actual_y = -actual_y - 1.0

            geomobj = gameworld.add_geom(size=(width, height), position=(actual_x, actual_y),
                               graphic_filename=filename,
                               tiles=(width, height),
                               no_collide=True,
                               no_detect=True,
                               draw_infront=True)


            geomobj.set_flip(doflip_h, doflip_v)

        xml_geoms = []

        # Get level geoms
        for layer in layers:
            if layer.attributes['inkscape:label'].value == "LEVEL_PLATFORMS":
                xml_geoms = layer.getElementsByTagName('image')


        for tag in xml_geoms:
            filename = os.path.split(tag.attributes['xlink:href'].value)[1]


            width = float(tag.attributes['width'].value)
            height = float(tag.attributes['height'].value)
            x = float(tag.attributes['x'].value)
            y = screen_height - (float(tag.attributes['y'].value) + height)
            doflip_h = False
            doflip_v = False
            try:
                moo = tag.attributes['transform'].value.encode('utf-8')
                moo = moo.replace('scale(','').replace(')', '').split(',')
                if float(moo[0]) < 0:
                    doflip_h = True

                if float(moo[1]) < 0:
                    y -= screen_height
                    y = screen_height - y - 1.0
                    doflip_v = True

            except Exception, e:
                pass

            actual_x = x + (width * 0.5)
            actual_y = y + (height * 0.5)

            if doflip_h:
                actual_x = -actual_x

            #if doflip_v:
                #actual_y = -actual_y - 1.0

            geomobj = gameworld.add_geom(size=(width, height), position=(actual_x, actual_y),
                               graphic_filename=filename,
                               tiles=(width, height),
                               no_collide=False,
                               no_detect=False,
                               is_platform=True)


            geomobj.set_flip(doflip_h, doflip_v)


        xml_geoms = []

        #get props
        for layer in layers:
            if layer.attributes['inkscape:label'].value == "LEVEL_PROPS":
                xml_geoms = layer.getElementsByTagName('image')


        for tag in xml_geoms:
            filename = os.path.split(tag.attributes['xlink:href'].value)[1]


            width = float(tag.attributes['width'].value)
            height = float(tag.attributes['height'].value)
            x = float(tag.attributes['x'].value)
            y = screen_height - (float(tag.attributes['y'].value) + height)
            doflip_h = False
            doflip_v = False
            try:
                moo = tag.attributes['transform'].value.encode('utf-8')
                moo = moo.replace('scale(','').replace(')', '').split(',')
                if float(moo[0]) < 0:
                    doflip_h = True

                if float(moo[1]) < 0:
                    y -= screen_height
                    y = screen_height - y - 1.0
                    doflip_v = True

            except Exception, e:
                pass




            actual_x = x + (width * 0.5)
            actual_y = y + (height * 0.5)

            if doflip_h:
                actual_x = -actual_x

            #if doflip_v:
                #actual_y = -actual_y - 1.0


            # do some clever prop finding here please
            prop = Prop
            if filename == 'small_box.png':
                prop = prop_small_box

            if filename == 'plank.png':
                prop = prop_plank

            if filename == 'ball.png':
                prop = prop_small_ball

            if filename == 'plank_break.png':
                prop = prop_break_plank

            params = {}
            try:
                moo = tag.attributes['params'].value
                for param in moo.split('&'):
                    key, value = param.split('=')
                    key = str(key)
                    value = str(value)
                    params[key] = value
            except:
                pass



            obj = gameworld.add_prop(prop_object=prop, position=(actual_x, actual_y), override_size=(width, height), params=params)
            obj.set_flip(doflip_h, doflip_v)


        xml_geoms = []

        #get Actors
        for layer in layers:
            if layer.attributes['inkscape:label'].value == "LEVEL_ACTORS":
                xml_geoms = layer.getElementsByTagName('image')


        for tag in xml_geoms:
            filename = os.path.split(tag.attributes['xlink:href'].value)[1]


            width = float(tag.attributes['width'].value)
            height = float(tag.attributes['height'].value)
            x = float(tag.attributes['x'].value)
            y = screen_height - (float(tag.attributes['y'].value) + height)
            doflip_h = False
            doflip_v = False
            try:
                moo = tag.attributes['transform'].value.encode('utf-8')
                moo = moo.replace('scale(','').replace(')', '').split(',')
                if float(moo[0]) < 0:
                    doflip_h = True

                if float(moo[1]) < 0:
                    y -= screen_height
                    y = screen_height - y - 1.0
                    doflip_v = True

            except Exception, e:
                pass




            actual_x = x + (width * 0.5)
            actual_y = y + (height * 0.5)

            if doflip_h:
                actual_x = -actual_x

            #if doflip_v:
                #actual_y = -actual_y - 1.0


            # do some clever prop finding here please
            actor = None
            if filename == 'UppetyBox.png':
                actor = UppetyBox

            elif filename == 'door_open.png':
                actor = Door

            elif filename == 'switch.png':
                actor = Switch

            elif filename == 'area_switch.png':
                actor = Area_Switch

            elif filename == 'door_h.png':
                actor = Door_h

            elif filename == 'spikes.png':
                actor = Spikes

            elif filename == 'end_trigger.png':
                actor = Level_End

            elif filename == 'warp_start.png':
                actor = Warp_Start

            elif filename == 'warp_end.png':
                actor = Warp_End

            elif filename == 'death_area_on_close.png':
                actor = Death_Area_Trigger

            elif filename == 'death_area.png':
                actor = Death_Area

            elif filename == 'lightpuzzlea.png':
                actor = LightPuzzle_A

            elif filename == 'lightpuzzlea_solver.png':
                actor = LightPuzzle_A_Solver

            elif filename == 'light_proxy.png':
                actor = LightProxy

            elif filename == 'multipass.png':
                actor = MultiSwitch

            elif filename == 'ghost_point.png':
                actor = GhostPoint

            elif filename == 'trap_door.png':
                actor = TrapDoor

            elif filename == 'trap_door_h.png':
                actor = TrapDoorH

            else:
                raise Exception, ('Unknown actor!', filename)

            params = {}
            try:
                moo = tag.attributes['params'].value
                for param in moo.split('&'):
                    key, value = param.split('=')
                    key = key.encode('utf-8')
                    value = value.encode('utf-8')

                    params[key] = value
            except Exception, e:
                #print 'params exception,', e
                pass



            #obj = gameworld.add_prop(prop_object=prop, position=(actual_x, actual_y), override_size=(width, height), params=params)
            obj = self.game_world.add_actor(actor_object=actor, position=(actual_x, actual_y), params=params, force_size=(width, height))
            obj.set_flip(doflip_h, doflip_v)

        xml_geoms = []

        # Get level lights
        for layer in layers:
            if layer.attributes['inkscape:label'].value == "LEVEL_LIGHTS":
                xml_geoms = layer.getElementsByTagName('image')


        for tag in xml_geoms:
            filename = os.path.split(tag.attributes['xlink:href'].value)[1]


            width = float(tag.attributes['width'].value)
            height = float(tag.attributes['height'].value)
            x = float(tag.attributes['x'].value)
            y = screen_height - (float(tag.attributes['y'].value) + height)
            doflip_h = False
            doflip_v = False
            try:
                moo = tag.attributes['transform'].value.encode('utf-8')
                moo = moo.replace('scale(','').replace(')', '').split(',')
                if float(moo[0]) < 0:
                    doflip_h = True

                if float(moo[1]) < 0:
                    y -= screen_height
                    y = screen_height - y - 1.0
                    doflip_v = True

            except Exception, e:
                pass






            color = [1.0, 1.0, 1.0, 1.0]

            try:
                moo = tag.attributes['style'].value
                #fill:#84eba3;fill-opacity:1
                colorhex = moo.split(';')[0].split(':')[1]
                colorhex = colorhex.encode('utf-8').replace('#', '')

                red = colorhex[0] + colorhex[1]
                green = colorhex[2] + colorhex[3]
                blue = colorhex[4] + colorhex[5]

                color = (float(int(red, 16)) / 255.0,
                         float(int(green, 16)) / 255.0,
                         float(int(blue, 16)) / 255.0,
                         1.0)

            except Exception, e:
                #print "Got exception,", e
                pass



            actual_x = x + (width * 0.5)
            actual_y = y + (height * 0.5)

            if doflip_h:
                actual_x = -actual_x

            #if doflip_v:
                #actual_y = -actual_y - 1.0


            obj = gameworld.add_light(position=(actual_x, actual_y), color=color,
                                texture=filename, size=(width, height))


            #obj.set_flip(doflip)
            obj.flip = (doflip_h, doflip_v)
        xml_geoms = []


        # Get level lights
        for layer in layers:
            if layer.attributes['inkscape:label'].value == "LEVEL_LIGHTS_NOAMBIANT":
                xml_geoms = layer.getElementsByTagName('image')


        for tag in xml_geoms:
            filename = os.path.split(tag.attributes['xlink:href'].value)[1]


            width = float(tag.attributes['width'].value)
            height = float(tag.attributes['height'].value)
            x = float(tag.attributes['x'].value)
            y = screen_height - (float(tag.attributes['y'].value) + height)
            doflip_h = False
            doflip_v = False
            try:
                moo = tag.attributes['transform'].value.encode('utf-8')
                moo = moo.replace('scale(','').replace(')', '').split(',')
                if float(moo[0]) < 0:
                    doflip_h = True

                if float(moo[1]) < 0:
                    y -= screen_height
                    y = screen_height - y - 1.0
                    doflip_v = True

            except Exception, e:
                pass






            color = [1.0, 1.0, 1.0, 1.0]

            try:
                moo = tag.attributes['style'].value
                #fill:#84eba3;fill-opacity:1
                colorhex = moo.split(';')[0].split(':')[1]
                colorhex = colorhex.encode('utf-8').replace('#', '')

                red = colorhex[0] + colorhex[1]
                green = colorhex[2] + colorhex[3]
                blue = colorhex[4] + colorhex[5]

                color = (float(int(red, 16)) / 255.0,
                         float(int(green, 16)) / 255.0,
                         float(int(blue, 16)) / 255.0,
                         1.0)

            except Exception, e:
                #print "Got exception,", e
                pass



            actual_x = x + (width * 0.5)
            actual_y = y + (height * 0.5)

            if doflip_h:
                actual_x = -actual_x

            #if doflip_v:
                #actual_y = -actual_y - 1.0

            obj = gameworld.add_light(position=(actual_x, actual_y), color=color,
                                texture=filename, size=(width, height), ambiant_affected=False)


            #obj.set_flip(doflip)
            obj.flip = (doflip_h, doflip_v)
        xml_geoms = []



    def __init__(self, renderlib, input):
        self.render = renderlib
        self.input = input

        if pygame.joystick.get_count():
            self.joystick = pygame.joystick.Joystick(0)
            self.joystick.init()
        else:
            self.joystick = None

        self.do_joy = False
        self.cam = Camera((0.0, 0.0))


    def pre_shutdown(self):
        self.input.remove_key(K_a, self.cb_walk, repeating=True)
        self.input.remove_key(K_d, self.cb_walk, repeating=True)
        self.input.remove_key(K_w, self.cb_jump, repeating=True)
        self.input.remove_key(K_LEFT, self.cb_walk, repeating=True)
        self.input.remove_key(K_RIGHT, self.cb_walk, repeating=True)
        self.input.remove_key(K_UP, self.cb_jump, repeating=True)
        self.input.remove_key(K_BACKSPACE, self.timeline_cb_reset_level)

        self.do_joy = False

    def shutdown(self):
        self.input.remove_key(K_a, self.cb_walk, repeating=True)
        self.input.remove_key(K_d, self.cb_walk, repeating=True)
        self.input.remove_key(K_w, self.cb_jump, repeating=True)
        self.input.remove_key(K_LEFT, self.cb_walk, repeating=True)
        self.input.remove_key(K_RIGHT, self.cb_walk, repeating=True)
        self.input.remove_key(K_UP, self.cb_jump, repeating=True)
        self.input.remove_key(K_BACKSPACE, self.timeline_cb_reset_level)

        self.game_world.shutdown()

        del(self.game_world)
        del(self.player)
        del(self.timeline)

        if self.joystick:
            self.joystick.quit()
            self.joystick = None


    def step(self):
        pos = self.player.get_position()
        pos = self.cam.move_to(pos)
        self.render.reset()
        self.render.set_screen_pos(-pos[0], -pos[1])
        self.render.set_screen_zoom(1.5)
        self.render.set_screen()
        self.timeline.step()

        if not self.do_reset:
            if not self.player.is_hurt:
                self.game_world.step()
                self.game_world.draw()

                if self.joystick and self.do_joy:
                    jx = self.joystick.get_axis(0)
                    jy = self.joystick.get_axis(1)

                    if jx > 0.0:
                        self.cb_walk(K_d)

                    if jx < 0.0:
                        self.cb_walk(K_a)

                    dojump = False
                    if jy != 0.0:
                        dojump = True

                    buttons = self.joystick.get_numbuttons()
                    for index in range(buttons):
                        if self.joystick.get_button(index):
                            dojump = True

                    if dojump:
                        self.cb_jump(K_w)

            else:
                self.do_reset = True
                self.pre_shutdown()
                self.timeline.add_action(time=0.0, length=2.0, relative=time_now,
                                     repeat_callback=True,
                                     callback=self.timeline_cb_fadeout_white)

                self.timeline.add_action(time=2.25, relative=time_now,
                                     callback=self.timeline_cb_reset_level)


        if self.game_world.end_level:
            self.timeline.add_action(time=0.0, length=2.0, relative=time_now,
                                     repeat_callback=True,
                                     callback=self.timeline_cb_fadeout_white)

            self.timeline.add_action(time=2.25, relative=time_now,
                                     callback=self.timeline_cb_end_level)
            self.do_reset = True

        if self.do_fade:
            self.draw_fs(fade=self.fade_value, blend=self.fade_blend)



        if self.reset_scene:
            self.start_level()

    def set_keys(self):
        self.input.add_key(K_a, self.cb_walk, repeating=True)
        self.input.add_key(K_d, self.cb_walk, repeating=True)
        self.input.add_key(K_w, self.cb_jump, repeating=True)
        self.input.add_key(K_BACKSPACE, self.timeline_cb_reset_level)

        self.do_joy = True

    def cb_skip(self, key):
        self.stop_scene = True

    def timeline_cb_fadein_white(self, position):
        self.fade_value = cosine_interpolate(1.0, 0.0, position)
        self.fade_blend = 'addictive'
        self.do_fade = True


    def timeline_cb_fadeout_white(self, position):
        self.fade_value = cosine_interpolate(0.0, 1.0, position)
        self.fade_blend = 'addictive'
        self.do_fade = True

    def timeline_cb_reset_level(self, position):
        self.reset_scene = True

    def timeline_cb_end_level(self, position):
        self.stop_scene = True



    def draw_fs(self, fade=1.0, blend='subtractive'):
        scale = self.render.base_scalar
        height = 10.0 * self.render.base_scalar[1]

        self.render.reset()
        self.render.enable(GL_BLEND)
        self.render.set_blend(blend)
        self.render.disable(GL_TEXTURE_2D)

        glBegin(GL_QUADS)

        glColor4fv((fade, fade, fade, 1.0))
        glVertex3f(-100.0, -height, -1.0)
        glVertex3f( 100.0, -height, -1.0)
        glVertex3f( 100.0, height, -1.0)
        glVertex3f(-100.0, height, -1.0)

        glEnd()

    def cb_walk(self, key):

        if key == K_a or key == K_LEFT:
            self.player.go_left()
        if key == K_d or key == K_RIGHT:
            self.player.go_right()

    def cb_jump(self, key):
        self.player.jump()

    def cb_recordcopy(self, key):
        if key == K_p:
            self.player.toggle_thread(1)
        if key == K_o:
            self.player.toggle_thread(2)

    def cb_playcopy(self, key):
        if key == K_l:
            self.player.spawn_thread(1)
        if key == K_k:
            self.player.spawn_thread(2)
